
package com.picfun.weatherboardslave3;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import com.picfun.weatherboardslave3.R;
import com.picfun.weatherboardslave3.USBAccessoryManager;
import com.picfun.weatherboardslave3.USBAccessoryManagerMessage;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;

public class WeatherBoardSlave3 extends Activity {
	// クラス定数の宣言定義
	private static Button cont1, cont2;
	private static TextView state1, state2, tempData, humiData, baroData;
	private static ProgressBar illumiData;	
	private static TextView text1, text2, text3;
	// USB関連クラス変数定義
	private USBAccessoryManager accessoryManager;
	private byte[] commandPacket = new byte[64];		// USB送受信バッファ
	private byte[] DOState = new byte[2];				// DO状態
	private double Ondo, Humi, Pres;	
	// USB制御コマンド定数定義 
	private static final byte USBAccessoryWhat = 0;
	private static final byte CONTROL			= 1;
	private static final byte STATUS			= 2;
	private static final byte TEMPARATURE		= 3;
	private static final byte HUMIDITY		= 4;
	private static final byte BAROMETER		= 5;
	private static final byte ILLUMINATION	= 6;
	private static final byte DISCONNECT 		= 0x7F;	

	//************ 最初に実行される関数 *******************/
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);		// タイトル削除
        // **** GUI画面表示 ****/       
        setContentView(R.layout.main);
        //コンポーネントの名称設定
        cont1    = (Button)this.findViewById(R.id.button1);
        cont2    = (Button)this.findViewById(R.id.button2);
        state1   = (TextView)this.findViewById(R.id.status1);
        state2   = (TextView)this.findViewById(R.id.status2); 
        tempData = (TextView)this.findViewById(R.id.temparature);
        humiData = (TextView)this.findViewById(R.id.humidity);
        baroData = (TextView)this.findViewById(R.id.barometer);
        illumiData = (ProgressBar) findViewById(R.id.illumination);	       
        text1    = (TextView)this.findViewById(R.id.textView1);        
        text2    = (TextView)this.findViewById(R.id.textView2);
        text3    = (TextView)this.findViewById(R.id.textView3);     
    	// USBアクセサリのインスタンス生成        
       	accessoryManager = new USBAccessoryManager(handler, USBAccessoryWhat);
        // ボタンイベント組み込み
        cont1.setOnClickListener((OnClickListener) new DO1_Control());
        cont2.setOnClickListener((OnClickListener) new DO2_Control());       
    }
    
    //******** アプリケーション遷移イベント関連メソッド **********/
	@Override
	public void onStart() {
		super.onStart();
		text1.setTextColor(Color.YELLOW);					// 初期状態
		text1.setText("ウェザーボード未接続");
	}
    @Override
    public void onResume() {
    	super.onResume();
 		text1.setTextColor(Color.GREEN);
		text1.setText("アプリケーション再起動");
	    accessoryManager.enable(this, getIntent());			// USB再接続実行     
    }
    @Override
    public void onPause() {
    	super.onPause();
		commandPacket[0] = DISCONNECT;						// 切り離し要求
		accessoryManager.write(commandPacket);	
    	text2.setTextColor(Color.YELLOW);
    	text2.setText("アプリケーション一旦停止");		
    	accessoryManager.disable(this);						// USB切り離し実行
    }
    
    //******** USBイベントハンドラ  **************/   
    private Handler handler = new Handler() {
    	@Override
     	/* USBライブラリ応答メッセージ処理 */
    	public void handleMessage(Message msg) {
			switch(((USBAccessoryManagerMessage)msg.obj).type) {
				case CONNECTED:													// 接続開始
					text1.setTextColor(Color.GREEN);
					text1.setText("USBホストが接続されました");						
					break;
				case READY:														// 接続後準備完了
					text2.setTextColor(Color.GREEN);
					text2.setText("USBホスト正常接続完了");
					break;
				case DISCONNECTED:												// 切り離された場合
					text3.setTextColor(Color.RED);
					text3.setText("USBホストが切り離されました");
					break;								
				case READ:
					if(accessoryManager.isConnected() == false) {				// 接続中か確認
						text3.setTextColor(Color.YELLOW);
						text3.setText("USBホスト接続なし、終了します！");
						return;													// 強制終了
					}
					Process();
					break;
				default:
					break;
			}
    	}
    };
    //****** 受信データ処理メソッド ******/
    private void Process(){
    	int temp;
		/* 受信バッファのデータ処理 */
		if(accessoryManager.available() > 2) {			// 受信データ残チェック
			accessoryManager.read(commandPacket);		// 受信データ取り出し64バイト
			// 受信コマンドごとの処理
			switch(commandPacket[0]) {
				case STATUS:					/* DI/DO状態の表示 */	
					temp = commandPacket[1] == 0x31 ? 0xFF00FF00:0xFFFF0000;
					state1.setBackgroundColor(temp);				// DI1背景色セット
					temp = commandPacket[2] == 0x31 ? 0xFF00FF00:0xFFFF0000;
					state2.setBackgroundColor(temp);				// DI2背景色セット	
					temp = commandPacket[3] == 0x30 ? 0xFF00FF00:0xFFFF0000;
					cont1.setBackgroundColor(temp);					// DO1背景色セット
					temp = commandPacket[3] == 0x30 ? 0:1;
					DOState[0] = (byte)temp;						// 状態更新
					temp = commandPacket[4] == 0x30 ? 0xFF00FF00:0xFFFF0000;
					cont2.setBackgroundColor(temp);					// DO2背景色セット
					temp = commandPacket[4] == 0x30 ? 0:1;
					DOState[1] = (byte)temp;						// 状態更新	
	    			break;
				case TEMPARATURE:				/* 温度をテキスト表示 */
					temp = (int)commandPacket[1]*256+(int)(commandPacket[2] & 0x7F);
					if((commandPacket[2] & 0x80) != 0)
						temp += 128; 
					Ondo = 0.01 * (double)temp - 39.7;
					tempData.setText(Double.toString(Ondo));
					break;	
				case HUMIDITY:					/* 湿度をテキスト表示 */						
					temp = (int)commandPacket[1]*256+(int)(commandPacket[2] & 0x7F);
					if((commandPacket[2] & 0x80) != 0)
						temp += 128; 							
					Humi = 0.0367 * (double)temp - 2.0468 - 0.0000016*temp*temp;
					Humi += (Ondo-25.0)*(0.01+0.00008*temp);		// 温度補正処理
					humiData.setText(Double.toString(Humi));
					break;
				case BAROMETER:					/* 気圧をテキスト表示 */
					Pres = (int)commandPacket[1]*256+(int)(commandPacket[2] & 0x7F);
					if((commandPacket[2] & 0x80) != 0)
						Pres += 128; 							
					baroData.setText(Double.toString(Pres/10));
					break;									
				case ILLUMINATION:				/* 照度をプログレスバーで表示　*/
					temp = (int)commandPacket[1]*256+(int)(commandPacket[2] & 0x7F);
					if((commandPacket[2] & 0x80) != 0)
						temp += 128;
					illumiData.setProgress(temp);
					break;
				default: break;
			}
		}								
    }
    
    //******* ボタンイベント処理サブクラス ******/
    class DO1_Control implements OnClickListener{
    	public void onClick(View v){
    		// 現在状態と反対の制御を実行する
    		commandPacket[0] = CONTROL;
	   		commandPacket[1] = 0x31;				// DO1   		
    		if(DOState[0]==0)   			
    			commandPacket[2] = 0x31;			// On
   			else
   				commandPacket[2] = 0x30;			// Off
    		accessoryManager.write(commandPacket); 	// 送信実行  
    	}
    }
    //****** ボタンイベント処理サブクラス ******/
    class DO2_Control implements OnClickListener{
      	public void onClick(View v){
    	// 現在状態と反対の制御を実行する
			commandPacket[0] = CONTROL;
	   		commandPacket[1] = 0x32;				// DO2			
			if(DOState[1]==0)
				commandPacket[2] = 0x31;			// On
			else
				commandPacket[2] = 0x30;			// Off	
			accessoryManager.write(commandPacket); 	// 送信実行  
      	}
    }
}

